@@ -14,20 +14,18 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Stef Walter <stef@thewalter.net>
  */
 
-#if !defined (__GCR_INSIDE_HEADER__) && !defined (GCR_COMPILATION)
-#error "Only <gcr/gcr.h> or <gcr/gcr-base.h> can be included directly."
-#endif
-
 #ifndef __GCR_SYSTEM_PROMPTER_H__
 #define __GCR_SYSTEM_PROMPTER_H__
 
+#if !defined (__GCR_INSIDE_HEADER__) && !defined (GCR_COMPILATION)
+#error "Only <gcr/gcr.h> can be included directly."
+#endif
+
 #include "gcr-prompt.h"
 #include "gcr-secret-exchange.h"
 #include "gcr-types.h"
@@ -87,6 +85,8 @@ void                   gcr_system_prompter_register                (GcrSystemPro
 void                   gcr_system_prompter_unregister              (GcrSystemPrompter *self,
                                                                     gboolean wait);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GcrSystemPrompter, g_object_unref)
+
 G_END_DECLS
 
 #endif /* __GCR_SYSTEM_PROMPTER_H__ */
