@@ -193,6 +193,13 @@ test_expiry_date (Test *test, gconstpointer unused)
 	g_date_time_unref (date);
 }
 
+static void
+test_version (Test *test, gconstpointer unused)
+{
+	gulong version = gcr_certificate_get_version (test->certificate);
+	g_assert_cmpuint (version, ==, 1);
+}
+
 static void
 test_serial_number (Test *test, gconstpointer unused)
 {
@@ -266,6 +273,38 @@ test_basic_constraints (Test *test,
 	g_assert (path_len == -1);
 }
 
+
+static void
+test_interface_elements (Test *test,
+                         gconstpointer unused)
+{
+	GList* sections = gcr_certificate_get_interface_elements (test->dsa_cert);
+	for (GList *l = sections; l != NULL; l = l->next) {
+		GcrCertificateSection *section = l->data;
+		GListModel *fields;
+
+		gcr_certificate_section_get_flags (section);
+		g_assert (gcr_certificate_section_get_label (section) != NULL);
+		fields = gcr_certificate_section_get_fields (section);
+		g_assert (fields != NULL);
+		g_assert (g_list_model_get_item_type (fields) == GCR_TYPE_CERTIFICATE_FIELD);
+		for (guint i = 0; i < g_list_model_get_n_items (fields); i++) {
+			GValue val = G_VALUE_INIT;
+			GType value_type;
+			GcrCertificateField *field = g_list_model_get_item (fields, i);
+			g_assert (gcr_certificate_field_get_label (field) != NULL);
+			value_type = gcr_certificate_field_get_value_type (field);
+			g_value_init (&val, value_type);
+			g_assert (gcr_certificate_field_get_value (field, &val));
+			g_value_unset (&val);
+			g_assert (gcr_certificate_field_get_section (field) == section);
+			g_object_unref (field);
+		}
+	}
+
+	g_list_free_full (sections, (GDestroyNotify) g_object_unref);
+}
+
 static void
 test_subject_alt_name (void)
 {
@@ -340,12 +379,14 @@ main (int argc, char **argv)
 	g_test_add ("/gcr/certificate/subject_raw", Test, NULL, setup, test_subject_raw, teardown);
 	g_test_add ("/gcr/certificate/issued_date", Test, NULL, setup, test_issued_date, teardown);
 	g_test_add ("/gcr/certificate/expiry_date", Test, NULL, setup, test_expiry_date, teardown);
+	g_test_add ("/gcr/certificate/version", Test, NULL, setup, test_version, teardown);
 	g_test_add ("/gcr/certificate/serial_number", Test, NULL, setup, test_serial_number, teardown);
 	g_test_add ("/gcr/certificate/fingerprint", Test, NULL, setup, test_fingerprint, teardown);
 	g_test_add ("/gcr/certificate/fingerprint_hex", Test, NULL, setup, test_fingerprint_hex, teardown);
 	g_test_add ("/gcr/certificate/key_size", Test, NULL, setup, test_certificate_key_size, teardown);
 	g_test_add ("/gcr/certificate/is_issuer", Test, NULL, setup, test_certificate_is_issuer, teardown);
 	g_test_add ("/gcr/certificate/basic_constraints", Test, NULL, setup, test_basic_constraints, teardown);
+	g_test_add ("/gcr/certificate/interface_elements", Test, NULL, setup, test_interface_elements, teardown);
 	g_test_add_func ("/gcr/certificate/subject_alt_name", test_subject_alt_name);
 	g_test_add_func ("/gcr/certificate/key_usage", test_key_usage);
 
